pub mod api;
pub mod display;
pub mod errors;
pub mod imageproc;

use crate::display::{EInkPanel, FakeEInk, Wrapper};
use crate::imageproc::{DitherMethod, DitheredImage};
use clap::{Parser, Subcommand};
use image::RgbImage;
use tracing::{error, info};

/// Display images on E-Ink Displays
#[derive(Debug, Parser)]
#[command(version, about)]
struct Cli {
    #[command(subcommand)]
    command: Command,
}

#[derive(Debug, Subcommand)]
enum Command {
    /// Load a single image
    Show,
    /// Display a test pattern
    Test,
    /// Start the HTTP server
    Serve,
}

#[tokio::main]
async fn main() -> anyhow::Result<()> {
    tracing_subscriber::fmt::init();
    let cli = Cli::parse();
    println!("CLI {cli:?}");

    if matches!(cli.command, Command::Show) {
        let img: RgbImage = image::ImageReader::open("image.png")?.decode()?.into();
        error!("HI");
        let mut display = FakeEInk {};

        let mut eink_buf = DitheredImage::default();
        let mut dither = DitherMethod::Atkinson.get_ditherer();

        dither.dither(&img, &mut eink_buf);
        display.display(&eink_buf)?;
    }
    if matches!(cli.command, Command::Test) {
        let mut display = Wrapper::new()?;

        display.test()?;
    }

    if matches!(cli.command, Command::Serve) {
        let display = FakeEInk {};

        let ctx = api::Context::new(Box::new(display));

        let app = api::router().with_state(ctx);
        let listener = tokio::net::TcpListener::bind("0.0.0.0:3000").await?;
        info!("Listening on 0.0.0.0:3000");
        axum::serve(listener, app).await?;
    }

    Ok(())
}
